  .datamapplot-progress-container {
    position: absolute;
    bottom: 0;
    left: 50%;
    transform: translate(-50%, 0);
    width: 512px;
    z-index: 100;
  }
  .datamapplot-progress-bar {
    width: 100%;
    background-color: #e0e0e044;  /* Changed to a light gray */
    padding: 3px;
    border-radius: 6px;
    box-shadow: inset 0 1px 3px rgba(128, 128, 128, .2);
    margin-bottom: 10px;
  }
  .datamapplot-progress-bar-fill {
    display: block;
    height: 12px;
    background-color: #659cef;
    border-radius: 4px;
    transition: width 500ms ease-in-out;
  }
  .datamapplot-progress-bar-text {
    color: white;
    text-align: center;
    line-height: 12px;
    font-size: 10px;
    overflow-x: visible;
    white-space: nowrap;
    vertical-align: top;
  }